@@ -4,7 +4,7 @@
 // found in the LICENSE file.
 
 mergeInto(LibraryManager.library, {
-  $FS__deps: ['__setErrNo', '$PATH', '$TTY', '$MEMFS',
+  $FS__deps: ['__setErrNo', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
 #if LibraryManager.has('library_idbfs.js')
     '$IDBFS',
 #endif
@@ -65,7 +65,7 @@ mergeInto(LibraryManager.library, {
     // paths
     //
     lookupPath: function(path, opts) {
-      path = PATH.resolve(FS.cwd(), path);
+      path = PATH_FS.resolve(FS.cwd(), path);
       opts = opts || {};
 
       if (!path) return { path: '', node: null };
@@ -116,7 +116,7 @@ mergeInto(LibraryManager.library, {
           var count = 0;
           while (FS.isLink(current.mode)) {
             var link = FS.readlink(current_path);
-            current_path = PATH.resolve(PATH.dirname(current_path), link);
+            current_path = PATH_FS.resolve(PATH.dirname(current_path), link);
 
             var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
             current = lookup.node;
@@ -679,7 +679,7 @@ mergeInto(LibraryManager.library, {
       return FS.mknod(path, mode, dev);
     },
     symlink: function(oldpath, newpath) {
-      if (!PATH.resolve(oldpath)) {
+      if (!PATH_FS.resolve(oldpath)) {
         throw new FS.ErrnoError({{{ cDefine('ENOENT') }}});
       }
       var lookup = FS.lookupPath(newpath, { parent: true });
@@ -720,12 +720,12 @@ mergeInto(LibraryManager.library, {
       // source must exist
       var old_node = FS.lookupNode(old_dir, old_name);
       // old path should not be an ancestor of the new path
-      var relative = PATH.relative(old_path, new_dirname);
+      var relative = PATH_FS.relative(old_path, new_dirname);
       if (relative.charAt(0) !== '.') {
         throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
       }
       // new path should not be an ancestor of the old path
-      relative = PATH.relative(new_path, old_dirname);
+      relative = PATH_FS.relative(new_path, old_dirname);
       if (relative.charAt(0) !== '.') {
         throw new FS.ErrnoError({{{ cDefine('ENOTEMPTY') }}});
       }
@@ -872,7 +872,7 @@ mergeInto(LibraryManager.library, {
       if (!link.node_ops.readlink) {
         throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
       }
-      return PATH.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
+      return PATH_FS.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
     },
     stat: function(path, dontFollow) {
       var lookup = FS.lookupPath(path, { follow: !dontFollow });
@@ -1204,7 +1204,17 @@ mergeInto(LibraryManager.library, {
       stream.stream_ops.allocate(stream, offset, length);
     },
     mmap: function(stream, buffer, offset, length, position, prot, flags) {
-      // TODO if PROT is PROT_WRITE, make sure we have write access
+      // User requests writing to file (prot & PROT_WRITE != 0).
+      // Checking if we have permissions to write to the file unless
+      // MAP_PRIVATE flag is set. According to POSIX spec it is possible
+      // to write to file opened in read-only mode with MAP_PRIVATE flag,
+      // as all modifications will be visible only in the memory of
+      // the current process.
+      if ((prot & {{{ cDefine('PROT_WRITE') }}}) !== 0
+          && (flags & {{{ cDefine('MAP_PRIVATE')}}}) === 0
+          && (stream.flags & {{{ cDefine('O_ACCMODE') }}}) !== {{{ cDefine('O_RDWR')}}}) {
+        throw new FS.ErrnoError({{{ cDefine('EACCES') }}});
+      }
       if ((stream.flags & {{{ cDefine('O_ACCMODE') }}}) === {{{ cDefine('O_WRONLY')}}}) {
         throw new FS.ErrnoError({{{ cDefine('EACCES') }}});
       }
@@ -1506,7 +1516,7 @@ mergeInto(LibraryManager.library, {
       return path;
     },
     absolutePath: function(relative, base) {
-      return PATH.resolve(base, relative);
+      return PATH_FS.resolve(base, relative);
     },
     standardizePath: function(path) {
       return PATH.normalize(path);
@@ -1658,12 +1668,12 @@ mergeInto(LibraryManager.library, {
       var success = true;
       if (typeof XMLHttpRequest !== 'undefined') {
         throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
-      } else if (Module['read']) {
+      } else if (read_) {
         // Command-line.
         try {
           // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
           //          read() will try to parse UTF8.
-          obj.contents = intArrayFromString(Module['read'](obj.url), true);
+          obj.contents = intArrayFromString(read_(obj.url), true);
           obj.usedBytes = obj.contents.length;
         } catch (e) {
           success = false;
@@ -1690,10 +1700,10 @@ mergeInto(LibraryManager.library, {
         var chunkOffset = idx % this.chunkSize;
         var chunkNum = (idx / this.chunkSize)|0;
         return this.getter(chunkNum)[chunkOffset];
-      }
+      };
       LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
         this.getter = getter;
-      }
+      };
       LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
         // Find length
         var xhr = new XMLHttpRequest();
@@ -1760,7 +1770,7 @@ mergeInto(LibraryManager.library, {
         this._length = datalength;
         this._chunkSize = chunkSize;
         this.lengthKnown = true;
-      }
+      };
       if (typeof XMLHttpRequest !== 'undefined') {
         if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
         var lazyArray = new LazyUint8Array();
@@ -1858,7 +1868,7 @@ mergeInto(LibraryManager.library, {
       Browser.init(); // XXX perhaps this method should move onto Browser?
       // TODO we should allow people to just pass in a complete filename instead
       // of parent and name being that we just join them anyways
-      var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
+      var fullname = name ? PATH_FS.resolve(PATH.join2(parent, name)) : parent;
       var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
       function processData(byteArray) {
         function finish(byteArray) {
